var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import * as React from 'react';
import styled from 'react-emotion';
export var NotieLevel;
(function (NotieLevel) {
    NotieLevel["INFO"] = "INFO";
    NotieLevel["ERROR"] = "ERROR";
    NotieLevel["SUCCESS"] = "SUCCESS";
    NotieLevel["WARN"] = "WARN";
    NotieLevel["FORCE"] = "FORCE";
    NotieLevel["CONFIRM"] = "CONFIRM";
})(NotieLevel || (NotieLevel = {}));
const colors = {
    [NotieLevel.SUCCESS]: '#13ce66',
    [NotieLevel.ERROR]: '#ff4949',
    [NotieLevel.INFO]: '#20a0ff',
    [NotieLevel.WARN]: '#f5be4b',
    [NotieLevel.CONFIRM]: '#20a0ff',
    [NotieLevel.FORCE]: '#20a0ff'
};
export const NOTIE_CLOSE_CLASS = `notie-${Math.random()
    .toString(36)
    .slice(2)}`;
export const NotieContainer = styled('dialog')({
    position: 'fixed',
    width: '100%',
    textAlign: 'center',
    fontSize: '14px',
    color: '#fff',
    margin: '0 auto',
    whiteSpace: 'nowrap',
    border: 'none',
    textOverflow: 'ellipsis',
    left: 0,
    top: 0,
    padding: 0,
    pointerEvents: 'none',
    '&[open]': {
        pointerEvents: 'auto'
    }
}, (props) => ({
    backgroundColor: props.level in colors ? colors[props.level] : '#444'
}));
NotieContainer.displayName = 'NotieContainer';
export const NotieMessage = styled('div')({
    padding: '20px'
});
NotieMessage.displayName = 'NotieMessage';
export const NotieChoices = styled('div')({
    display: 'flex',
    width: '100%'
});
NotieChoices.displayName = 'NotieChoices';
export const NotieButton = styled('button')({
    padding: '20px',
    flexGrow: 1,
    border: 'none',
    textAlign: 'center',
    cursor: 'pointer',
    outline: 'none'
}, (props) => ({
    backgroundColor: props.level in colors ? colors[props.level] : '#444'
}));
NotieButton.displayName = 'NotieButton';
export const NotieContext = React.createContext({
    message: '',
    level: NotieLevel.INFO,
    okBtnText: '',
    cancelBtnText: '',
    success: () => null,
    error: () => null,
    warn: () => null,
    info: () => null,
    confirm: () => __awaiter(this, void 0, void 0, function* () { return Promise.resolve(''); }),
    force: () => __awaiter(this, void 0, void 0, function* () { return Promise.resolve(''); })
});
const DEFAULT_STATE = {
    message: '',
    level: NotieLevel.INFO,
    okBtnText: 'OK',
    cancelBtnText: 'Cancel'
};
let alreadyLoaded = false;
export const DEFAULT_TIMEOUT = 5000;
export class Notie extends React.Component {
    constructor(props) {
        super(props);
        this.dialog = null;
        this.timeout = null;
        this.showNotie = (settings) => {
            if (this.forced)
                return;
            if (this.timeout) {
                clearTimeout(this.timeout);
                this.timeout = null;
            }
            if (this.dialog && this.dialog.open) {
                this.hideNotie(() => {
                    this.showNotie(settings);
                });
                return;
            }
            this.setState((state) => ({
                message: settings.message,
                level: settings.level,
                okBtnText: settings.okBtnText || state.okBtnText,
                cancelBtnText: settings.cancelBtnText || state.cancelBtnText
            }), () => {
                if (!this.dialog)
                    return;
                if (this.forced) {
                    this.dialog.showModal();
                }
                else {
                    this.dialog.show();
                    this.timeout = window.setTimeout(() => {
                        this.hideNotie();
                        this.timeout = null;
                    }, settings.ttl || DEFAULT_TIMEOUT);
                }
                this.dialog.animate([
                    { transform: 'translate3d(0, -106%, 0)' },
                    { transform: 'translate3d(0, 0, 0)' }
                ], {
                    duration: 300,
                    direction: 'normal'
                });
            });
        };
        this.hideNotie = (callback) => {
            if (!this.dialog)
                return;
            const a = this.dialog.animate([
                { transform: 'translate3d(0, 0, 0)' },
                { transform: 'translate3d(0, -106%, 0)' }
            ], {
                duration: 300,
                direction: 'normal'
            });
            a.onfinish = () => {
                if (this.dialog)
                    this.dialog.close();
                if (typeof callback === 'function')
                    callback();
                this.setState(DEFAULT_STATE);
            };
        };
        this.handleYes = () => {
            this.hideNotie(this.confirmResolve);
        };
        this.handleNo = () => {
            this.hideNotie(this.confirmReject);
        };
        this.handleDismiss = () => {
            if (this.forced)
                return;
            if (this.timeout) {
                clearTimeout(this.timeout);
                this.timeout = null;
            }
            if (this.dialog && this.dialog.open) {
                this.hideNotie();
            }
        };
        this.rootRef = (c) => {
            this.dialog = c;
        };
        this.showSuccess = (message, ttl) => {
            this.showNotie(Object.assign({}, DEFAULT_STATE, { level: NotieLevel.SUCCESS, message,
                ttl }));
        };
        this.showError = (message, ttl) => {
            this.showNotie(Object.assign({}, DEFAULT_STATE, { level: NotieLevel.ERROR, message,
                ttl }));
        };
        this.showWarn = (message, ttl) => {
            this.showNotie(Object.assign({}, DEFAULT_STATE, { level: NotieLevel.WARN, message,
                ttl }));
        };
        this.showInfo = (message, ttl) => {
            this.showNotie(Object.assign({}, DEFAULT_STATE, { level: NotieLevel.INFO, message,
                ttl }));
        };
        this.showConfirm = (message, props) => __awaiter(this, void 0, void 0, function* () {
            // tslint:disable-next-line:no-inferred-empty-object-type
            return new Promise((resolve, reject) => {
                this.confirmResolve = resolve;
                this.confirmReject = reject;
                this.showNotie(Object.assign({}, DEFAULT_STATE, { level: NotieLevel.CONFIRM }, props, { message }));
            });
        });
        this.showForce = (message, props) => __awaiter(this, void 0, void 0, function* () {
            // tslint:disable-next-line:no-inferred-empty-object-type
            return new Promise((resolve, reject) => {
                this.confirmResolve = resolve;
                this.confirmReject = reject;
                this.showNotie(Object.assign({}, DEFAULT_STATE, { level: NotieLevel.FORCE }, props, { message }));
            });
        });
        this.state = Object.assign({}, DEFAULT_STATE);
        if (alreadyLoaded) {
            throw new Error('<Notie /> is already loaded somwhere in your code. Please include it only once.');
        }
        alreadyLoaded = true;
    }
    componentWillUnmount() {
        if (this.timeout) {
            clearTimeout(this.timeout);
            this.timeout = null;
        }
        if (alreadyLoaded)
            alreadyLoaded = false;
    }
    get forced() {
        const { level } = this.state;
        return level === NotieLevel.CONFIRM || level === NotieLevel.FORCE;
    }
    render() {
        const { message, level } = this.state;
        return (React.createElement(NotieContext.Provider, { value: Object.assign({}, this.state, { confirm: this.showConfirm, error: this.showError, info: this.showInfo, success: this.showSuccess, warn: this.showWarn, force: this.showForce }) },
            this.props.children,
            React.createElement(NotieContainer, { innerRef: this.rootRef, level: level, onClick: this.handleDismiss },
                React.createElement(NotieMessage, null, message),
                this.forced && (React.createElement(NotieChoices, null,
                    React.createElement(NotieButton, { level: level === NotieLevel.FORCE
                            ? NotieLevel.ERROR
                            : NotieLevel.SUCCESS, onClick: this.handleYes }, this.state.okBtnText),
                    level === NotieLevel.CONFIRM && (React.createElement(NotieButton, { level: NotieLevel.ERROR, onClick: this.handleNo }, this.state.cancelBtnText)))))));
    }
}
